data {
    int N;
    array[N] real X;
    array[N] real Y;
    array[N] real Z;
}
parameters {
    real vmuX;
    real vmuY;
    real vmuZ;
    real<lower = 0.0> vsgmX;
    real<lower = 0.0> vsgmY;
    real<lower = 0.0> vsgmZ;
    cholesky_factor_corr[3] cL;
}
transformed parameters {
    real muX;
    real muY;
    real muZ;
    real RhoXY;
    real RhoXZ;
    real RhoYZ;
    real<lower = 0.0> sgmX;
    real<lower = 0.0> sgmY;
    real<lower = 0.0> sgmZ;
    corr_matrix[3] Corr;
    cov_matrix[3] Cov;

    muX = vmuX * 100.0;
    muY = vmuY * 100.0;
    muZ = vmuZ * 100.0;
    sgmX = 0.01 + vsgmX * 100;
    sgmY = 0.01 + vsgmY * 100;
    sgmZ = 0.01 + vsgmZ * 100;
    //
    //     Correlation matrix
    //
    Corr = cL * cL';
    //
    //      Covariance matrix
    //
    Cov[1][1] = sgmX*sgmX;
    Cov[1][2] = Corr[1][2]*sgmX*sgmY;
    Cov[1][3] = Corr[1][3]*sgmX*sgmZ;
    Cov[2][1] = Corr[2][1]*sgmY*sgmX;
    Cov[2][2] = sgmY*sgmY;
    Cov[2][3] = Corr[2][3]*sgmY*sgmZ;
    Cov[3][1] = Corr[3][1]*sgmX*sgmZ;
    Cov[3][2] = Corr[3][2]*sgmY*sgmZ;
    Cov[3][3] = sgmZ*sgmZ;
    //
    //       Correlations
    //
    RhoXY = Corr[1][2];
    RhoXZ = Corr[1][3];
    RhoYZ = Corr[2][3];
}
model {
    vmuX ~ normal(0.0, 10.0);
    vmuY ~ normal(0.0, 10.0);
    vmuZ ~ normal(0.0, 10.0);
    vsgmX ~ exponential(0.1);
    vsgmY ~ exponential(0.1);
    vsgmZ ~ exponential(0.1);
    cL ~ lkj_corr_cholesky(1.0);
    for (i in 1:N) {
        [X[i], Y[i], Z[i]] ~ multi_normal([muX, muY, muZ], Cov);
    }
}
