data {
    int N;
    array[N] real X;
    array[N] real Y;
    array[N] real Z;
}
parameters {
    real vmu1;
    real vmu2;
    real<lower = 0.0> vsgm1;
    real<lower = 0.0> vsgm2;
    real<lower = 0.0> vsgm_x;
    real<lower = 0.0> vsgm_y;
    real<lower = -1.0, upper = 1.0> rho; 
    real Ax;
    real Bx;
    real Ay;
    real By;
}
transformed parameters {
    real mu1;
    real mu2;
    real<lower = 0.0> sgm1;
    real<lower = 0.0> sgm2;
    real<lower = 0.0> sgm_x;
    real<lower = 0.0> sgm_y;
    array[N] vector[2] XY;
    
    mu1 = vmu1 * 100;
    mu2 = vmu2 * 100;
    sgm1 = 0.001 + vsgm1 * 1000;
    sgm2 = 0.001 + vsgm2 * 1000;
    sgm_x = 0.001 + vsgm_x * 1000;
    sgm_y = 0.001 + vsgm_y * 1000;
    
    for (i in 1:N) {
        XY[i][1] = X[i] - (Bx + Ax * Z[i]);    //   Residual
        XY[i][2] = Y[i] - (By + Ay * Z[i]);    //   Residual
    }
}
model {
    vmu1 ~ normal(0.0, 1.0);
    vmu2 ~ normal(0.0, 1.0);
    vsgm1 ~ exponential(0.1);
    vsgm2 ~ exponential(0.1);
    rho ~ uniform(-1.0, 1.0);     //    Partial correlation
    Ax ~ normal(0.0, 10.0);
    Ay ~ normal(0.0, 10.0);
    Bx ~ normal(0.0, 10.0);
    By ~ normal(0.0, 10.0);
    vsgm_x ~ exponential(0.1);
    vsgm_y ~ exponential(0.1);
    for (i in 1:N) {
        X[i] ~ normal(Bx + Ax * Z[i], sgm_x);
        Y[i] ~ normal(By + Ay * Z[i], sgm_y);
        //
        //      The model for estimation of the partial correlation rho
        //
        XY[i] ~ multi_normal([mu1, mu2], [[sgm1*sgm1, rho*sgm1*sgm2], [rho*sgm2*sgm1, sgm2*sgm2]]);
    }
}
